@@ -73,35 +73,46 @@ static long pv__calc_eta(const long long so_far, const long long total,
 
 /*
  * Given a long double value, it is divided or multiplied by the ratio until
- * a value in the range 1.0 to 999.999... is found. The string "prefix" to
+ * a value in the range 1.0 to 999.999... is found.  The string "prefix" to
  * is updated to the corresponding SI prefix.
  *
  * If "is_bytes" is 1, then the second byte of "prefix" is set to "i" to
- * denote MiB etc (IEEE1541). Thus "prefix" should be at least 3 bytes long
+ * denote MiB etc (IEEE1541).  Thus "prefix" should be at least 3 bytes long
  * (to include the terminating null).
  *
  * Submitted by Henry Gebhardt <hsggebhardt@googlemail.com> and then
- * modified. Further changed after input from Thomas Rachel.
+ * modified.  Further changed after input from Thomas Rachel; changed still
+ * further after Debian bug #706175.
  */
 static void pv__si_prefix(long double *value, char *prefix,
 			  const long double ratio, int is_bytes)
 {
-	static char *pfx = NULL;
-	static char const *pfx_middle = NULL;
+	static char *pfx_000 = NULL;	 /* kilo, mega, etc */
+	static char *pfx_024 = NULL;	 /* kibi, mibi, etc */
+	static char const *pfx_middle_000 = NULL;
+	static char const *pfx_middle_024 = NULL;
+	char *pfx;
+	char const *pfx_middle;
 	char const *i;
 	long double cutoff;
 
-	if (NULL == pfx) {
-		pfx = _("yzafpnum kMGTPEZY");
+	if (NULL == pfx_000) {
+		pfx_000 = _("yzafpnum kMGTPEZY");
+		pfx_middle_000 = strchr(pfx_000, ' ');
 	}
 
-	if (NULL == pfx_middle) {
-		/*
-		 * We can't assign this in the declaration above because
-		 * that wouldn't be constant, so we do it here.
-		 */
-		pfx_middle = strchr(pfx, ' ');
+	if (NULL == pfx_024) {
+		pfx_024 = _("yzafpnum KMGTPEZY");
+		pfx_middle_024 = strchr(pfx_024, ' ');
 	}
+
+	pfx = pfx_000;
+	pfx_middle = pfx_middle_000;
+	if (is_bytes) {
+		pfx = pfx_024;
+		pfx_middle = pfx_middle_024;
+	}
+
 	i = pfx_middle;
 
 	prefix[0] = ' ';		    /* Make the prefix start blank. */
