@@ -1,7 +1,7 @@
 /*
  * Functions for transferring between file descriptors.
  *
- * Copyright 2013 Andrew Wood, distributed under the Artistic License 2.0.
+ * Copyright 2014 Andrew Wood, distributed under the Artistic License 2.0.
  */
 
 #include "pv-internal.h"
@@ -50,15 +50,25 @@ static int pv__transfer_read(pvstate_t state, int fd,
 	long orig_offset;
 	long skip_offset;
 	ssize_t nread;
+#ifdef HAVE_SPLICE
+	size_t bytes_to_splice;
+#endif				/* HAVE_SPLICE */
 
 	bytes_can_read = state->buffer_size - state->read_position;
 
 #ifdef HAVE_SPLICE
 	state->splice_used = 0;
-	if ((!state->linemode) && (fd != state->splice_failed_fd)
+	if ((!state->linemode) && (!state->no_splice)
+	    && (fd != state->splice_failed_fd)
 	    && (0 == state->to_write)) {
-		nread = splice(fd, NULL, STDOUT_FILENO, NULL, allowed,
-			       SPLICE_F_MORE);
+		if (state->rate_limit || allowed != 0)
+			bytes_to_splice = allowed;
+		else
+			bytes_to_splice = bytes_can_read;
+
+		nread = splice(fd, NULL, STDOUT_FILENO, NULL,
+			       bytes_to_splice, SPLICE_F_MORE);
+
 		state->splice_used = 1;
 		if ((nread < 0) && (EINVAL == errno)) {
 			state->splice_failed_fd = fd;
@@ -68,6 +78,8 @@ static int pv__transfer_read(pvstate_t state, int fd,
 			 */
 		} else if (nread > 0) {
 			state->written = nread;
+		} else if ((-1 == nread) && (EAGAIN == errno)) {
+			/* nothing read yet - do nothing */
 		} else {
 			/* EOF might not really be EOF, it seems */
 			state->splice_used = 0;
@@ -341,8 +353,20 @@ static int pv__transfer_write(pvstate_t state, int fd,
 			    (char *) (state->transfer_buffer +
 				      state->write_position - 1);
 
-			while ((ptr = strchr((char *) (ptr + 1), '\n')))
-				++lines;
+			if (state->null) {
+				for (ptr++;
+				     ptr -
+				     (char *) state->transfer_buffer -
+				     state->write_position < nwrote;
+				     ptr++) {
+					if (*ptr == '\0')
+						++lines;
+				}
+			} else {
+				while ((ptr =
+					strchr((char *) (ptr + 1), '\n')))
+					++lines;
+			}
 
 			*lineswritten += lines;
 			state->transfer_buffer[state->write_position +
@@ -352,6 +376,42 @@ static int pv__transfer_write(pvstate_t state, int fd,
 		state->write_position += nwrote;
 		state->written += nwrote;
 
+		/*
+		 * If we're monitoring the output, update our copy of the
+		 * last few bytes we've written.
+		 */
+		if (((state->components_used & PV_DISPLAY_OUTPUTBUF) != 0)
+		    && (nwrote > 0)) {
+			long new_portion_length, old_portion_length;
+
+			new_portion_length = nwrote;
+			if (new_portion_length > state->lastoutput_length)
+				new_portion_length =
+				    state->lastoutput_length;
+
+			old_portion_length =
+			    state->lastoutput_length - new_portion_length;
+
+			/*
+			 * Make room for the new portion.
+			 */
+			if (old_portion_length > 0) {
+				memmove(state->lastoutput_buffer,
+					state->lastoutput_buffer +
+					new_portion_length,
+					old_portion_length);
+			}
+
+			/*
+			 * Copy the new data in.
+			 */
+			memcpy(state->lastoutput_buffer +
+			       old_portion_length,
+			       state->transfer_buffer +
+			       state->write_position - new_portion_length,
+			       new_portion_length);
+		}
+
 		/*
 		 * If we've written all the data in the buffer, reset the
 		 * read pointer to the start, and if the input file is at
@@ -558,7 +618,7 @@ long pv_transfer(pvstate_t state, int fd, int *eof_in, int *eof_out,
 	 * In line mode, only write up to and including the last newline,
 	 * so that we're writing output line-by-line.
 	 */
-	if ((state->to_write > 0) && (state->linemode)) {
+	if ((state->to_write > 0) && (state->linemode) && !(state->null)) {
 		/*
 		 * Guillaume Marcais: use strrchr to find last \n
 		 */
